@@ -3,16 +3,18 @@
 
 __all__ = ['Bitbucket', ]
 
+from urlparse import parse_qs
 import json
+import re
 
-from requests import Request
-from requests import Session
-from requests import Response
+from requests import Request, Session
+from requests_oauthlib import OAuth1
+import requests
 
+from issue import Issue
 from repository import Repository
 from service import Service
 from ssh import SSH
-from issue import Issue
 
 
 class Bitbucket(object):
@@ -22,7 +24,7 @@ class Bitbucket(object):
     def __init__(self, username='', password='', repo_name_or_slug=''):
         self.username = username
         self.password = password
-        self.repo_slug = repo_name_or_slug.lower().replace(r'[^a-z0-9_-]+', '-')
+        self.repo_slug = repo_name_or_slug
         self.repo_tree = {}
 
         self.repository = Repository(self)
@@ -30,6 +32,12 @@ class Bitbucket(object):
         self.ssh = SSH(self)
         self.issue = Issue(self)
 
+        self.access_token = None
+        self.access_token_secret = None
+        self.consumer_key = None
+        self.consumer_secret = None
+        self.oauth = None
+
     #  ===================
     #  = Getters/Setters =
     #  ===================
@@ -37,6 +45,8 @@ class Bitbucket(object):
     @property
     def auth(self):
         """ Return credentials for current Bitbucket user. """
+        if self.oauth:
+            return self.oauth
         return (self.username, self.password)
 
     @property
@@ -78,16 +88,82 @@ class Bitbucket(object):
 
     @repo_slug.setter
     def repo_slug(self, value):
-        if isinstance(value, basestring):
-            value = unicode(value)
-        self._repo_slug = value.lower().replace(r'[^a-z0-9_-]+', '-')
         if value is None:
             self._repo_slug = None
+        else:
+            if isinstance(value, basestring):
+                value = unicode(value)
+            value = value.lower()
+            self._repo_slug = re.sub(r'[^a-z0-9_-]+', '-', value)
 
     @repo_slug.deleter
     def repo_slug(self):
         del self._repo_slug
 
+    #  ========================
+    #  = Oauth authentication =
+    #  ========================
+
+    def authorize(self, consumer_key, consumer_secret, callback_url=None, access_token=None, access_token_secret=None):
+        """ Call this with your consumer key, secret and callback URL, to generate a token for verification. """
+        self.consumer_key = consumer_key
+        self.consumer_secret = consumer_secret
+
+        if not access_token and not access_token_secret:
+            if not callback_url:
+                return (False, "Callback URL required")
+            oauth = OAuth1(
+                consumer_key,
+                client_secret=consumer_secret,
+                callback_uri=callback_url)
+            r = requests.post(self.url('REQUEST_TOKEN'), auth=oauth)
+            if r.status_code == 200:
+                creds = parse_qs(r.content)
+
+                self.access_token = creds.get('oauth_token')[0]
+                self.access_token_secret = creds.get('oauth_token_secret')[0]
+            else:
+                return (False, r.content)
+        else:
+            self.finalize_oauth(access_token, access_token_secret)
+
+        return (True, None)
+
+    def verify(self, verifier, consumer_key=None, consumer_secret=None, access_token=None, access_token_secret=None):
+        """ After converting the token into verifier, call this to finalize the authorization. """
+        # Stored values can be supplied to verify
+        self.consumer_key = consumer_key or self.consumer_key
+        self.consumer_secret = consumer_secret or self.consumer_secret
+        self.access_token = access_token or self.access_token
+        self.access_token_secret = access_token_secret or self.access_token_secret
+
+        oauth = OAuth1(
+            self.consumer_key,
+            client_secret=self.consumer_secret,
+            resource_owner_key=self.access_token,
+            resource_owner_secret=self.access_token_secret,
+            verifier=verifier)
+        r = requests.post(self.url('ACCESS_TOKEN'), auth=oauth)
+        if r.status_code == 200:
+            creds = parse_qs(r.content)
+        else:
+            return (False, r.content)
+
+        self.finalize_oauth(creds.get('oauth_token')[0], creds.get('oauth_token_secret')[0])
+        return (True, None)
+
+    def finalize_oauth(self, access_token, access_token_secret):
+        """ Called internally once auth process is complete. """
+        self.access_token = access_token
+        self.access_token_secret = access_token_secret
+
+        # Final OAuth object
+        self.oauth = OAuth1(
+            self.consumer_key,
+            client_secret=self.consumer_secret,
+            resource_owner_key=self.access_token,
+            resource_owner_secret=self.access_token_secret)
+
     #  ======================
     #  = High lvl functions =
     #  ======================
@@ -118,7 +194,9 @@ class Bitbucket(object):
                     pass
             return (True, text)
         elif status >= 300 and status < 400:
-            return (False, 'Unauthorized access, '
+            return (
+                False,
+                'Unauthorized access, '
                 'please check your credentials.')
         elif status >= 400 and status < 500:
             return (False, 'Service not found.')
@@ -165,7 +243,7 @@ class Bitbucket(object):
     #  = URLs =
     #  ========
     URLS = {
-        'BASE': 'https://api.bitbucket.org/1.0/%s',
+        'BASE': 'https://bitbucket.org/!api/1.0/%s',
         # Get user profile and repos
         'GET_USER': 'users/%(username)s/',
         # Search repo
@@ -173,4 +251,8 @@ class Bitbucket(object):
         # Get tags & branches
         'GET_TAGS': 'repositories/%(username)s/%(repo_slug)s/tags/',
         'GET_BRANCHES': 'repositories/%(username)s/%(repo_slug)s/branches/',
+
+        'REQUEST_TOKEN': 'oauth/request_token/',
+        'AUTHENTICATE': 'oauth/authenticate?oauth_token=%(token)s',
+        'ACCESS_TOKEN': 'oauth/access_token/'
     }
